import argparse
import copy
import logging
import time
from datetime import datetime
from argparse import Namespace
from threading import Thread
from typing import Optional, Union, Dict, Any, Tuple, List

from pytorch_lightning.utilities import rank_zero_only
from pytorch_lightning.loggers import LightningLoggerBase

from dldstraining.core.reference import Ref
from dldstraining.core.utils import AsyncDispatch

logger = logging.getLogger(__name__)


class DLDSLogger(LightningLoggerBase):

    def __init__(self, model, trainer_model, name: str = "default", version: Optional[Union[int, str]] = None,
                 **kwargs) -> None:
        super().__init__(**kwargs)
        self.model = model
        self.trainer_model = trainer_model
        self.client = self.model.client
        self.data_model = self.model.platform_model
        self.__version: Union[int, str] = version if version else 0
        self.__name: str = name

        self.upload_metrics = {'statistics': {}, 'results': {}}

        self.upload_model_statistics_dispatch: AsyncDispatch[Tuple[Any, Any, Any, Any]] = AsyncDispatch(
            self.upload_model_statistics, Ref(self.trainer_model, 'running'))

        self.upload_inference_results_dispatch: AsyncDispatch[Tuple[Any, Any, Any, Any, Any]] = AsyncDispatch(
            self.upload_inference_results, Ref(self.trainer_model, 'running'), finish_batch_when_terminated=False)

        self.upload_inference_results_dispatch.start()
        self.upload_model_statistics_dispatch.start()

    @property
    def experiment(self):
        """Return the experiment object associated with this logger."""
        return self

    @property
    def name(self) -> str:
        """Return the experiment name."""
        return self.__name

    @property
    def version(self) -> Union[int, str]:
        """Return the experiment version."""
        return self.__version

    @rank_zero_only
    def log_hyperparams(self, params: Namespace) -> None:
        pass

    @rank_zero_only
    def log_dlds_train_metrics(self, metrics):
        train_dict = {'train': {'scalars': {}}}
        for key in metrics.keys():
            update_value = metrics[key]
            if key in train_dict['train']['scalars'].keys():
                old_value = train_dict['train']['scalars'][key]
                new_value = metrics[key]
                update_value = 0.1 * new_value + 0.9 * old_value
            train_dict['train']['scalars'].update({key: float(update_value)})

        self.upload_metrics['statistics'].update(train_dict)
        #XXX: no save here?

    @rank_zero_only
    def log_dlds_validation_metrics(self, metrics) -> None:
        assert 'statistics' in metrics.keys() and 'results' in metrics.keys()
        self.upload_metrics['statistics'].update(metrics['statistics'])
        self.upload_metrics['results'] = metrics['results'] # detection boxes
        self.save()

    @rank_zero_only
    def log_metrics(self, metrics: Dict[str, Any], step) -> None:
        """
        Metrics is a dictionary of metric names and values

        :param metrics:
        :param step:
        :return:
        """
        pass

    def save(self):


        statistics_available: bool = 'eval' in self.upload_metrics['statistics'] \
                                     and 'train' in self.upload_metrics['statistics']
        results_available = any(self.upload_metrics['results'])

        if not (statistics_available or results_available):
            return

        logger.info('Triggering model statistics upload...')
        current_step = copy.deepcopy(self.model.trainer.global_step)

        if statistics_available:
            argument: List[Any] = copy.deepcopy([self.upload_metrics['statistics'],
                                                 datetime.utcnow().isoformat(), current_step,
                                                 self.model.platform_model['id']])
            self.upload_model_statistics_dispatch.update(tuple(argument))
            self.upload_metrics['statistics'].clear()



        if results_available:
            argument: List[Any] = copy.deepcopy([self.upload_metrics['results'],
                                                 datetime.utcnow().isoformat(), current_step,
                                                 self.model.platform_model['id']])
            self.upload_inference_results_dispatch.update(tuple(argument))
            self.upload_metrics['results'].clear()

        logger.info('Model statistics upload triggered.')

    def upload_model_statistics(self, statistics, timestamp, iteration, model_id):
        logger.info('Uploading model statistics...')
        try:
            self.client.upload_model_statistics(statistics, timestamp, iteration, model_id)
            logger.info('Successfully uploaded model statistics.')
        except Exception as e:
            logger.error(f'Uploading model statistics failed: {e}')

        return True

    def upload_inference_results(self, *args):
        logger.info('Uploading inference results...')
        try:
            self.client.upload_inference_results(*args)
            logger.info('Successfully uploaded inference results.')
        except Exception as e:
            logger.error(f'Uploading inference results failed: {e}')
            raise e from None


        return True

    @rank_zero_only
    def finalize(self, status) -> None:
        """
        Optional. Any code that needs to be run after training finishes goes here
        :param status:
        """
        pass
