#  Copyright (c) 2022 Data Spree GmbH - All Rights Reserved
#  Unauthorized copying of this file, via any medium is strictly prohibited.
#  Proprietary and confidential.

import logging
import os
import sys
import time
import json
from logging.handlers import RotatingFileHandler
import urllib3

import click
from dataspree.platform_sdk.client import Client

logger = logging.getLogger(__name__)


@click.command()
@click.option('--model_id', type=int, default=None,
              help='ID of this worker.')
@click.option('--keep_up_time', type=int, default=None,
              help='Enable debugging information.')
@click.option('--debug', is_flag=True,
              help='Enable debug mode which sets the default log level and changes error handling.')
@click.option('--only_warnings', is_flag=True,
              help='Enable only log message of type warning or higher.')
@click.option('--worker_id', type=int, default=None,
              help='Optional ID of the training worker.')
@click.option('--platform_token', type=str, default=None,
              help='Optional auth. token for the training worker.')
@click.option('--dataset_directory', type=str, default=None,
              help='Optional dataset directory for the training worker.')
@click.option('--platform_host', type=str, default='api.vision.data-spree.com',
              help='Optional dataset directory for the training worker.')
def cli(model_id, keep_up_time, debug:bool, only_warnings:bool,
        worker_id, platform_token, dataset_directory, platform_host) -> None:
    running = True
    log_level = logging.INFO
    if debug:
        log_level = logging.DEBUG
    if only_warnings:
        log_level = logging.WARNING

    logging.basicConfig(level=log_level, format='%(asctime)s %(name)s %(levelname)-6s %(message)s')
    logging.getLogger('aiohttp.access').setLevel(logging.WARNING)

    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    training_dir = os.path.join(Client.dataspree_dir(), 'training')
    log_dir = os.path.join(training_dir, 'logs')
    os.makedirs(log_dir, exist_ok=True)
    log_file = os.path.join(log_dir, 'training.log')
    rotating_file_handler = RotatingFileHandler(log_file, maxBytes=25 * 1024 * 1024, backupCount=10)
    log_formatter = logging.Formatter('%(asctime)s %(name)s %(levelname)s %(message)s')
    rotating_file_handler.setFormatter(log_formatter)
    logging.getLogger().addHandler(rotating_file_handler)

    while running:
        from dataspree.platform_sdk.worker import Worker
        try:
            if worker_id is not None and platform_token is not None and dataset_directory is not None:
                logger.info('Writing worker config file using the provided CLI arguments.')
                worker_config_file_path = os.path.join(training_dir, 'worker.json')
                worker_config = {
                    'auth_token': platform_token,
                    'platform_host': platform_host,
                    'dataset_dir': dataset_directory,
                    'worker_id': worker_id
                }
                with open(worker_config_file_path, 'w') as f:
                    json.dump(worker_config, f, indent=2)

            Worker.start(model_id, keep_up=keep_up_time is None, keep_up_time=keep_up_time, debug=debug)
            if keep_up_time is not None:
                break
        except KeyboardInterrupt:
            running = False
        except Exception as e:
            if debug:
                raise e
            logger.error(e)
            time.sleep(1)

    logger.info('Worker stopped.')
    sys.exit(0)


if __name__ == '__main__':
    cli()
